<?php

namespace Drupal\mollie\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Class WebhookController.
 *
 * @package Drupal\mollie\Controller
 */
class WebhookController extends ControllerBase {

  /**
   * HTTP kernel.
   *
   * @var \Symfony\Component\HttpKernel\HttpKernelInterface
   */
  protected $httpKernel;

  /**
   * Request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * RedirectController constructor.
   *
   * @param HttpKernelInterface $httpKernel
   *   HTTP kernel.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   Request stack.
   */
  public function __construct(HttpKernelInterface $httpKernel, RequestStack $requestStack) {
    $this->httpKernel = $httpKernel;
    $this->requestStack = $requestStack;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('http_kernel'),
      $container->get('request_stack')
    );
  }

  /**
   * Updates the status of a payment.
   *
   * @param string $context
   *   The type of context that requested the payment.
   * @param string $context_id
   *   The ID of the context that requested the payment.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   Response that redirects to the appropriate place.
   */
  public function invokeHook(string $context, string $context_id): Response {
    // Return a 200 to let Mollie know that we have received and handled
    // the request.
    $httpCode = 200;

    switch ($context) {
      // TODO: Abstract this so that it is only known in Mollie for Drupal Commerce.
      case 'mollie_commerce':
        // Fetch transaction ID sent by Mollie.
        $id = $this->requestStack->getCurrentRequest()->get('id');

        // We do a sub request to Drupal Commerce to handle payment and order
        // status updates there.
        $url = Url::fromRoute(
          'commerce_payment.notify',
          [
            'commerce_payment_gateway' => 'mollie',
          ]
        );
        $subRequest = Request::create(
          $url->toString(),
          'POST',
          ['mollie_transaction_id' => $id, 'order_id' => $context_id]
        );
        try {
          return $this->httpKernel
            ->handle($subRequest, HttpKernelInterface::SUB_REQUEST);
        }
        catch (\Exception $e) {
          watchdog_exception('mollie', $e);
          $httpCode = 500;
        }
        break;

      default:
        // We do not store Mollie transaction data in Drupal so we do not need
        // to update anything by default.
        break;
    }

    return new Response('', $httpCode);
  }

}
